@@ -12,7 +12,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import enum
 import os
+import re
 
 from .. import mlog
 from .. import mesonlib, build
@@ -24,6 +26,10 @@ from ..interpreter import CustomTargetHolder
 from ..interpreterbase import permittedKwargs, FeatureNewKwargs
 from ..dependencies import ExternalProgram
 
+class ResourceCompilerType(enum.Enum):
+    windres = 1
+    rc = 2
+
 class WindowsModule(ExtensionModule):
 
     def detect_compiler(self, compilers):
@@ -32,11 +38,61 @@ class WindowsModule(ExtensionModule):
                 return compilers[l]
         raise MesonException('Resource compilation requires a C or C++ compiler.')
 
+    def _find_resource_compiler(self, state):
+        # FIXME: Does not handle `native: true` executables, see
+        # See https://github.com/mesonbuild/meson/issues/1531
+
+        if hasattr(self, '_rescomp'):
+            return self._rescomp
+
+        rescomp = None
+        if state.environment.is_cross_build():
+            # If cross compiling see if windres has been specified in the
+            # cross file before trying to find it another way.
+            bins = state.environment.cross_info.config['binaries']
+            rescomp = ExternalProgram.from_bin_list(bins, 'windres')
+
+        if not rescomp or not rescomp.found():
+            if 'WINDRES' in os.environ:
+                # Pick-up env var WINDRES if set. This is often used for
+                # specifying an arch-specific windres.
+                rescomp = ExternalProgram('windres', command=os.environ.get('WINDRES'), silent=True)
+
+        if not rescomp or not rescomp.found():
+            # Take windres from the config file after the environment, which is
+            # in keeping with the expectations on unix-like OSes that
+            # environment variables trump config files.
+            bins = state.environment.config_info.binaries
+            rescomp = ExternalProgram.from_bin_list(bins, 'windres')
+
+        if not rescomp or not rescomp.found():
+            comp = self.detect_compiler(state.compilers)
+            if comp.id == 'msvc' or comp.id == 'clang-cl':
+                rescomp = ExternalProgram('rc', silent=True)
+            else:
+                rescomp = ExternalProgram('windres', silent=True)
+
+        if not rescomp.found():
+            raise MesonException('Could not find Windows resource compiler')
+
+        for (arg, match, type) in [
+                ('/?', '^.*Microsoft.*Resource Compiler.*$', ResourceCompilerType.rc),
+                ('--version', '^.*GNU windres.*$', ResourceCompilerType.windres),
+        ]:
+            p, o, e = mesonlib.Popen_safe(rescomp.get_command() + [arg])
+            m = re.search(match, o, re.MULTILINE)
+            if m:
+                mlog.log('Windows resource compiler: %s' % m.group())
+                self._rescomp = (rescomp, type)
+                break
+        else:
+            raise MesonException('Could not determine type of Windows resource compiler')
+
+        return self._rescomp
+
     @FeatureNewKwargs('windows.compile_resources', '0.47.0', ['depend_files', 'depends'])
     @permittedKwargs({'args', 'include_directories', 'depend_files', 'depends'})
     def compile_resources(self, state, args, kwargs):
-        comp = self.detect_compiler(state.compilers)
-
         extra_args = mesonlib.stringlistify(kwargs.get('args', []))
         wrc_depend_files = extract_as_list(kwargs, 'depend_files', pop = True)
         wrc_depends = extract_as_list(kwargs, 'depends', pop = True)
@@ -49,33 +105,24 @@ class WindowsModule(ExtensionModule):
                 raise MesonException('Resource include dirs should be include_directories().')
         extra_args += get_include_args(inc_dirs)
 
-        if comp.id == 'msvc':
-            rescomp = ExternalProgram('rc', silent=True)
-            res_args = extra_args + ['/nologo', '/fo@OUTPUT@', '@INPUT@']
+        rescomp, rescomp_type = self._find_resource_compiler(state)
+        if rescomp_type == ResourceCompilerType.rc:
+            # RC is used to generate .res files, a special binary resource
+            # format, which can be passed directly to LINK (apparently LINK uses
+            # CVTRES internally to convert this to a COFF object)
             suffix = 'res'
+            res_args = extra_args + ['/nologo', '/fo@OUTPUT@', '@INPUT@']
         else:
+            # ld only supports object files, so windres is used to generate a
+            # COFF object
+            suffix = 'o'
+            res_args = extra_args + ['@INPUT@', '@OUTPUT@']
+
             m = 'Argument {!r} has a space which may not work with windres due to ' \
                 'a MinGW bug: https://sourceware.org/bugzilla/show_bug.cgi?id=4933'
             for arg in extra_args:
                 if ' ' in arg:
                     mlog.warning(m.format(arg))
-            rescomp = None
-            # FIXME: Does not handle `native: true` executables, see
-            # https://github.com/mesonbuild/meson/issues/1531
-            if state.environment.is_cross_build():
-                # If cross compiling see if windres has been specified in the
-                # cross file before trying to find it another way.
-                cross_info = state.environment.cross_info
-                rescomp = ExternalProgram.from_cross_info(cross_info, 'windres')
-            if not rescomp or not rescomp.found():
-                # Pick-up env var WINDRES if set. This is often used for
-                # specifying an arch-specific windres.
-                rescomp = ExternalProgram(os.environ.get('WINDRES', 'windres'), silent=True)
-            res_args = extra_args + ['@INPUT@', '@OUTPUT@']
-            suffix = 'o'
-        if not rescomp.found():
-            raise MesonException('Could not find Windows resource compiler {!r}'
-                                 ''.format(rescomp.get_path()))
 
         res_targets = []
 
@@ -88,35 +135,38 @@ class WindowsModule(ExtensionModule):
             if hasattr(src, 'held_object'):
                 src = src.held_object
 
-            res_kwargs = {
-                'output': '@BASENAME@.' + suffix,
-                'input': [src],
-                'command': [rescomp] + res_args,
-                'depend_files': wrc_depend_files,
-                'depends': wrc_depends,
-            }
-
             if isinstance(src, str):
-                name = 'file {!r}'.format(os.path.join(state.subdir, src))
+                name_format = 'file {!r}'
+                name = os.path.join(state.subdir, src)
             elif isinstance(src, mesonlib.File):
-                name = 'file {!r}'.format(src.relative_name())
+                name_format = 'file {!r}'
+                name = src.relative_name()
             elif isinstance(src, build.CustomTarget):
                 if len(src.get_outputs()) > 1:
                     raise MesonException('windows.compile_resources does not accept custom targets with more than 1 output.')
 
-                name = 'target {!r}'.format(src.get_id())
+                name_format = 'target {!r}'
+                name = src.get_id()
             else:
                 raise MesonException('Unexpected source type {!r}. windows.compile_resources accepts only strings, files, custom targets, and lists thereof.'.format(src))
 
             # Path separators are not allowed in target names
             name = name.replace('/', '_').replace('\\', '_')
 
+            res_kwargs = {
+                'output': name + '_@BASENAME@.' + suffix,
+                'input': [src],
+                'command': [rescomp] + res_args,
+                'depend_files': wrc_depend_files,
+                'depends': wrc_depends,
+            }
+
             # instruct binutils windres to generate a preprocessor depfile
-            if comp.id != 'msvc':
+            if rescomp_type == ResourceCompilerType.windres:
                 res_kwargs['depfile'] = res_kwargs['output'] + '.d'
                 res_kwargs['command'] += ['--preprocessor-arg=-MD', '--preprocessor-arg=-MQ@OUTPUT@', '--preprocessor-arg=-MF@DEPFILE@']
 
-            res_targets.append(build.CustomTarget('Windows resource for ' + name, state.subdir, state.subproject, res_kwargs))
+            res_targets.append(build.CustomTarget('Windows resource for ' + name_format.format(name), state.subdir, state.subproject, res_kwargs))
 
         add_target(args)
 
